@@ -32,7 +32,9 @@ test_that("mixed logit model gives the same", {
   skip_if_not_installed("RcppArmadillo")
   skip_on_macOS()
 
-  Rcpp::sourceCpp(system.file("mlogit-ex.cpp", package = "psqn"))
+  reset_info <- compile_cpp_file("mlogit-ex.cpp")
+  on.exit(reset_compile_cpp_file(reset_info), add = TRUE)
+  setwd(reset_info$old_wd)
   optimizer <- get_mlogit_optimizer(sim_dat, max_threads = 2L)
 
   val <- c(beta, sapply(sim_dat, function(x) x$u))
